from .kor.resident_registration import ResidentRegistration
from .kor.old_registration_registration import OldResidentRegistration
from .kor.arc import ARC
from .util import alias_of

NationalID = alias_of(ResidentRegistration)
