from .col.unique_persional_id import UniquePersonalID
from .util import alias_of

NationalID = alias_of(UniquePersonalID)
"""alias of UniquePersonalID"""
NUIP = alias_of(UniquePersonalID)
"""alias of UniquePersonalID"""
