"""
The idnumbers package enables your application to parse, validate against to different specs. For example, it supports
[national ID](idnumbers/nationalid.html) for parsing and validating all citizen/resident ID numbers among different
countries.
"""
