from .fin.personal_id import PersonalIdentityCode
from .util import alias_of

NationalID = alias_of(PersonalIdentityCode)
"""alias of PersonalIdentityCode"""
HETU = alias_of(PersonalIdentityCode)
"""alias of PersonalIdentityCode"""
