from .alb.identity_number import IdentityNumber
from .util import alias_of

NationalID = alias_of(IdentityNumber)
"""alias of IdentityNumber"""
NISH = alias_of(IdentityNumber)
"""alias of IdentityNumber"""
NIPT = alias_of(IdentityNumber)
"""alias of IdentityNumber"""
NID = alias_of(IdentityNumber)
"""alias of IdentityNumber"""
