from .dnk.personal_id import PersonalIdentityNumber
from .util import alias_of

NationalID = alias_of(PersonalIdentityNumber)
"""alias of PersonalIdentityNumber"""
CPR = alias_of(PersonalIdentityNumber)
"""alias of PersonalIdentityNumber"""
